; ---------------------------------------------
; Sound Effects Driver for the NGPC v1.0
; by Ivan Mackintosh 15 July 01
;
; ---------------------------------------------

        cpu z80	
        org 0h

LEFT_CH_CTRL	EQU 04001h
RIGHT_CH_CTRL	EQU 04000h
FIRST_TXFER	EQU 10000000b
TONE_FREQ1	EQU 00000000b
TONE_VOL1	EQU 00010000b
TONE_FREQ2	EQU 00100000b
TONE_VOL2	EQU 00110000b
TONE_FREQ3	EQU 01000000b
TONE_VOL3	EQU 01010000b
NOISE_CTRL  EQU 01100000b
NOISE_VOL   EQU 01110000b



Start:
   ld sp, 0ff0h
   jp StartOfCode

; ---------------------------------------------
; TLCS900 Interface
; A series of booleans to control the Z80 + the
; InputSound Buffer
; ---------------------------------------------
               org 10h

VBPulse:       db 0 ; Pulsed from the VBInt of the TLCS900
PlayBuffer:    db 0 ; Plays the buffer directly
LoadPreset:    db 0 ; Copies to the buffer to a preset value (0-31)
PlayPreset:    db 0 ; Plays from a preset
StopAll        db 0 ; Stops all sound effects

               org 20h

InputSoundSize EQU         19
InputSound:    db 0        ; Channel 0,1,2,3,4 (0 and 4 are noise)
               db 0        ; length	
	       db 0	   ; repeat
               dw 0        ; initial tone
               dw 0        ; tone step
               db 0        ; tone step speed
               db 0        ; OWB (Boundaries - Off, Wrap, Bounce)
               dw 0        ; Lower limit
               dw 0        ; Upper limit
               db 0        ; initial vol
               db 0        ; vol step
               db 0        ; vol step speed
               db 0        ; Vol OWB
               db 0        ; Lower limit
               db 0        ; Upper limit

;
; *** buffer offset equates for above InputSound
;

; copied from buffer (having all of the +1's means that I can
; add a new parameter into the middle without having to renumber
; all of the subsequent ones
CH_CHANNEL	EQU 0
CH_LENGTH	EQU 1 
CH_REPEAT	EQU 2 
CH_INIT_TONE	EQU 3 
CH_TONE_STEP	EQU 5 
CH_TONE_SPEED	EQU 7 
CH_TONE_OWB	EQU 8 
CH_TONE_LLIMIT	EQU 9 
CH_TONE_ULIMIT	EQU 11
CH_INIT_VOL	EQU 13
CH_VOL_STEP	EQU 14
CH_VOL_SPEED	EQU 15
CH_VOL_OWB	EQU 16
CH_VOL_LLIMIT	EQU 17
CH_VOL_ULIMIT	EQU 18

; addition variables
CH_TICK_COUNT	EQU 19
CH_TONE_COUNT	EQU 20
CH_VOL_COUNT	EQU 21
CH_RUNNING     EQU 22
CH_START_TONE  EQU 23
CH_START_VOL   EQU 25
CH_START_TSTEP EQU 26
CH_START_VSTEP EQU 28


; ---------------------------------------------
; Z80 Sound Driver
; The code starts here
; ---------------------------------------------
StartOfCode:
   ; ensure all running variables are reset
   ld a, 0
   ld iy, NoiseBuf
   ld (iy+CH_RUNNING),a
   ld iy, Ch1Buf
   ld (iy+CH_RUNNING),a
   ld iy, Ch2Buf
   ld (iy+CH_RUNNING),a
   ld iy, Ch3Buf
   ld (iy+CH_RUNNING),a

MainLoop:
; poll for commands from the TLCS
   ld a, (VBPulse)
   cp a, 0
   jr nz, ProcessSoundEffects
EndProcessSoundEffects
   ld a, (PlayBuffer)
   cp a, 0
   jp nz, DoLoadChannel
EndLoadChannel:
   ld a, (LoadPreset)
   cp a, 0
   jp nz, DoLoadPreset
EndLoadPreset:
   ld a, (PlayPreset)
   cp a, 0
   jp nz, DoPlayPreset
EndPlayPreset:
   ld a, (StopAll)
   cp a, 0
   jp nz, DoStopAll
EndStopAll:
   jr MainLoop   

; process any outstanding sound effects
ProcessSoundEffects:
   ld a, 0
   ld (VBPulse), a

   ld a, 0
   call PlayChannel
   ld a, 1
   call PlayChannel
   ld a, 2
   call PlayChannel
   ld a, 3
   call PlayChannel
   jp EndProcessSoundEffects



; ---------------------------------------------
; Load Channels
; Copies all of the input sound buffer data to 
; a private copy used for channel 1 that can/is
; altered by the driver. Also initialises local
; variables.
; ---------------------------------------------
DoLoadChannel:
   ; clear command
   ld a, 0
   ld (PlayBuffer), a

   ; copy buffer to private space for this channel
   ld a, (InputSound) ; get channel number
   and a, 3           ; ensure it is in range (noise4 = 0)

   ; set the dest buffer address

DLC_SWITCH_A
   cp a, 0
   jr nz, DLC_CASE_1
   ld ix, NoiseBuf
   jr DLC_END_SWITCH
DLC_CASE_1
   cp a, 1
   jr nz, DLC_CASE_2
   ld ix, Ch1Buf
   jr DLC_END_SWITCH
DLC_CASE_2
   cp a, 2
   jr nz, DLC_CASE_3
   ld ix, Ch2Buf
   jr DLC_END_SWITCH
DLC_CASE_3
   ld ix, Ch3Buf
DLC_END_SWITCH

   ; set the source buffer
   ld iy, InputSound

   ; copy the buffer
   call CopyAndInitSoundBuffer
   jp EndLoadChannel



; ---------------------------------------------
; DoLoadPreset
; Copies the input buffer to a preset buffer
; specified by the command 1-31
; ---------------------------------------------
DoLoadPreset:
   ; bound check on limit
   and a,31
   
   ; find buffer start (multiply preset number by 32 and
   ; add the buffer start to it
   ld e,a
   ld a,0
   sla e ;x2 - max = 62
   sla e ;x4 - max = 124
   sla e ;x8 - max = 248
   sla e ;x16 - max = 496, roll carry to a
   adc a, 0
   sla a ;x32
   sla e ;x32, roll carry to a
   adc a, 0
   ld d, a ; de = a x 32

   ; all because "add de, Presets" doesn't work :-(
   ld hl, Presets
   add hl, de		; point de to appropriate destination preset
   push hl
   pop ix

   ld iy, InputSound	; source of copy
   ld a, 0
   
DLPCopy:
   ld b, (iy)
   ld (ix), b
   inc ix
   inc iy
   inc a
   cp a, InputSoundSize
   jr nz, DLPCopy

   ; copy complete now reset interface value so the TLCS knows
   ; it can add another one if it wants
   ld a, 0
   ld (LoadPreset), a

   jp EndLoadPreset


; ---------------------------------------------
; DoPlayPreset
; Copies the specified preset buffer to a channel
; buffer. The preset buffer is specified in the 
; command and the channel is stored in the preset
; ---------------------------------------------
DoPlayPreset:
   ; bound check 
   and a, 31

   ; set the source buffer

   ; find buffer start (multiply preset number by 32 and
   ; add the buffer start to it
   ld e,a
   ld a,0
   sla e ;x2 - max = 62
   sla e ;x4 - max = 124
   sla e ;x8 - max = 248
   sla e ;x16 - max = 496, roll carry to a
   adc a, 0
   sla a ;x32
   sla e ;x32, roll carry to a
   adc a, 0
   ld d, a ; de = a x 32
   
   ld hl, Presets
   add hl, de
   push hl ; copy hl to iy
   pop iy

   ; copy buffer to private space for this channel
   ld a, (iy+CH_CHANNEL) ; get channel number
   and a, 3              ; ensure it is in range (noise4 = 0)

   ; set the dest buffer address

DPP_SWITCH_A
   cp a, 0
   jr nz, DPP_CASE_1
   ld ix, NoiseBuf
   jr DPP_END_SWITCH
DPP_CASE_1
   cp a, 1
   jr nz, DPP_CASE_2
   ld ix, Ch1Buf
   jr DPP_END_SWITCH
DPP_CASE_2
   cp a, 2
   jr nz, DPP_CASE_3
   ld ix, Ch2Buf
   jr DPP_END_SWITCH
DPP_CASE_3
   ld ix, Ch3Buf
DPP_END_SWITCH

   ; copy the buffer
   call CopyAndInitSoundBuffer

   ; clear command
   ld a, 0
   ld (PlayPreset), a

   jp EndPlayPreset


; ---------------------------------------------
; DoStopAll
; Stops the playing of all channels
; ---------------------------------------------
DoStopAll:
   ld a, 0
   call DoStopChannel
   ld a, 1
   call DoStopChannel
   ld a, 2
   call DoStopChannel
   ld a, 3
   call DoStopChannel

   jp EndStopAll



; ---------------------------------------------
; CopyAndInitSoundBuffer
; Given input buffer and output buffer address
; Copies from one to the other and initialises
; additional variables
; Inputs ix = dest buffer + vars.
;        iy = input buffer
; ---------------------------------------------
CopyAndInitSoundBuffer:
   push ix ; save dest buffer base

   ld a, 0
CISBCopy:
   ld b, (iy)
   ld (ix), b
   inc iy
   inc ix
   inc a
   cp a, InputSoundSize
   jr nz, CISBCopy
    
   ; initialise buffer variables
   pop ix ; get dest buffer

   ld a, 0
   ld (ix+CH_TICK_COUNT), a
   ld (ix+CH_TONE_COUNT), a
   ld (ix+CH_VOL_COUNT), a

   ; store duplicate vars for Repeat
   ld a, (ix+CH_INIT_VOL)
   ld (ix+CH_START_VOL), a
   ld a, (ix+CH_VOL_STEP)
   ld (ix+CH_START_VSTEP), a

   ld a, (ix+CH_INIT_TONE)
   ld (ix+CH_START_TONE), a
   ld a, (ix+CH_INIT_TONE+1)
   ld (ix+CH_START_TONE+1), a
   ld a, (ix+CH_TONE_STEP)
   ld (ix+CH_START_TSTEP), a
   ld a, (ix+CH_TONE_STEP+1)
   ld (ix+CH_START_TSTEP+1), a

   ; set running flag
   ld a, 1
   ld (ix+CH_RUNNING), a
   ret


; -------------------------------------------------
; PlayChannel
; Plays the channel as specified by the 'a' reg.
; -------------------------------------------------
PlayChannel:

   ; set up the iy register to point to the correct
   ; channel buffer
PC_SWITCH_A
   cp a, 0
   jr nz, PC_CASE_1
   ld iy, NoiseBuf
   ld d, TONE_FREQ3 ; noise uses tone3 (right)
   ld e, NOISE_VOL
   ld ix, RIGHT_CH_CTRL
   jr PC_END_SWITCH
PC_CASE_1
   cp a, 1
   jr nz, PC_CASE_2
   ld iy, Ch1Buf
   ld d, TONE_FREQ1
   ld e, TONE_VOL1
   ld ix, LEFT_CH_CTRL
   jr PC_END_SWITCH
PC_CASE_2
   cp a, 2
   jr nz, PC_CASE_3
   ld iy, Ch2Buf
   ld d, TONE_FREQ2
   ld e, TONE_VOL2
   ld ix, LEFT_CH_CTRL
   jr PC_END_SWITCH
PC_CASE_3
   ld iy, Ch3Buf
   ld d, TONE_FREQ3
   ld e, TONE_VOL3
   ld ix, LEFT_CH_CTRL
PC_END_SWITCH

   ; check if there is anything to do (is the channel running?)
   ld a, (iy+CH_RUNNING) 
   cp a, 0
   jp z, PlayChannel_End

; --------- Noise Setup ---------
   
   ; are we doing noise?
   ld a, (iy+CH_CHANNEL)
   and a, 3
   cp a, 0
   jr nz, PC_EndNoiseSetup

   ; noise 3 or noise 7?
   ld a, (iy+CH_CHANNEL)
   cp a, 0
   jr nz, PC_NS_1
   ld a, 3
   jr PC_NS_2
PC_NS_1
   ld a, 7
PC_NS_2
   or a, NOISE_CTRL
   or a, FIRST_TXFER
   ld (ix), a

PC_EndNoiseSetup:


; --------- Volume + Tone Setup ---------

   ; invert the volume
   ld a, (iy+CH_INIT_VOL)
   ld b, a
   ld a, 15
   sub a, b
   or a, FIRST_TXFER
   or a, e
   ld (ix), a

   ; convert from xxxxxx9876543210
   ; to xxxx3210 and xx987654

   ld c, (iy+CH_INIT_TONE)
   ld a, c
   and a, 15
   or a, FIRST_TXFER
   or a, d
   ld (ix), a        ; first byte

   ld a, (iy+CH_INIT_TONE+1)
   and a, 3
   ld b, a
   ld a, (iy+CH_INIT_TONE)
   srl a
   srl b 
   jr nc, PC_1
   or a, 128
PC_1:
   srl a
   srl b
   jr nc, PC_2
   or a, 128
PC_2:
   srl a
   srl a
	ld (ix), a        ; second byte


; --------- Update Tone ---------

   ; if ToneSpeed = 0 do nothing
   ld a, (iy+CH_TONE_SPEED)
   cp a, 0
   jr z, PC_EndUpdateTone

   ; if ToneCount = ToneSpeed then change Tone
   ld b, a
   ld a, (iy+CH_TONE_COUNT)
   inc a
   ld (iy+CH_TONE_COUNT), a
   cp a, b
   jr nz, PC_EndUpdateTone
   ld a, 0
   ld (iy+CH_TONE_COUNT), a

   ; if OWB = 0 then process as normal
   ld a, (iy+CH_TONE_OWB)
   cp a, 0
   jr z, PC_UT_1
   call ProcessToneOWB
   jr PC_EndUpdateTone

PC_UT_1
   ; add the values together, And with 3ffh and store the new tone
   ld c, (iy+CH_TONE_STEP)
   ld b, (iy+CH_TONE_STEP+1)
   ld l, (iy+CH_INIT_TONE)
   ld h, (iy+CH_INIT_TONE+1)
   add hl, bc
   ld a, h
   and a, 3
   ld h, a
   ld (iy+CH_INIT_TONE), l
   ld (iy+CH_INIT_TONE+1), h

PC_EndUpdateTone:


; --------- Update Vol ---------

   ; if VolSpeed = 0 do nothing
   ld a, (iy+CH_VOL_SPEED)
   cp a, 0
   jr z, PC_EndUpdateVol

   ; if VolCount = VolSpeed then change vol
   ld b, a
   ld a, (iy+CH_VOL_COUNT)
   inc a
   ld (iy+CH_VOL_COUNT), a
   cp a, b
   jr nz, PC_EndUpdateVol
   ld a, 0
   ld (iy+CH_VOL_COUNT), a
   
   ; if OWB = then process as normal
   ld a, (iy+CH_VOL_OWB)   
   cp a, 0
   jr z, PC_UV_1
   call ProcessVolOWB
   jr PC_EndUpdateVol

PC_UV_1
   ; add values together, And with 0fh and store
   ld a, (iy+CH_INIT_VOL)
   ld b, a
   ld a, (iy+CH_VOL_STEP)
   add a, b
   and a, 0fh
   ld (iy+CH_INIT_VOL), a

PC_EndUpdateVol:

; --------- Update Length ---------

   ; now check length
   ld a, (iy+CH_TICK_COUNT)
   ld b, a
   ld a, (iy+CH_LENGTH)
   cp a, b
   jr nz, PC_LengthNotReached
   
   ; length reached - terminate sound
   ld a, (iy+CH_CHANNEL)
   call DoStopChannel
   call CheckForRepeat
   jr PC_EndUpdateLength

PC_LengthNotReached:

   ; update tick count
   ld a, (iy+CH_TICK_COUNT)
   inc a
   ld (iy+CH_TICK_COUNT), a

PC_EndUpdateLength:


PlayChannel_End:
   ret


; -------------------------------------------------
; CheckForRepeat
; Called when the length is reached to reinitialise
; if repeat is required
; inputs iy = channel buffer
; -------------------------------------------------
CheckForRepeat:
   ld a, (iy+CH_REPEAT)
   cp a, 0
   jr z, CheckForRepeatEnd

   ; at this point we have to reinitialise
   ld a, 0
   ld (iy+CH_TICK_COUNT), a
   ld (iy+CH_TONE_COUNT), a
   ld (iy+CH_VOL_COUNT), a
   ld a, 1
   ld (iy+CH_RUNNING), a
   ld a, (iy+CH_START_VOL)
   ld (iy+CH_INIT_VOL), a
   ld a, (iy+CH_START_VSTEP)
   ld (iy+CH_VOL_STEP), a
   ld a, (iy+CH_START_TONE)
   ld (iy+CH_INIT_TONE), a
   ld a, (iy+CH_START_TONE+1)
   ld (iy+CH_INIT_TONE+1), a
   ld a, (iy+CH_START_TSTEP)
   ld (iy+CH_TONE_STEP), a
   ld a, (iy+CH_START_TSTEP+1)
   ld (iy+CH_TONE_STEP+1), a
   
   ld a, (iy+CH_REPEAT)
   cp a, 0ffh ; infinite
   jr z, CheckForRepeatEnd
   
   dec a
   ld (iy+CH_REPEAT), a

CheckForRepeatEnd:   
   ret


; -------------------------------------------------
; ProcessToneOWB
; If the OWB setting is set to 1 (wrap) then the
; upper limit is checked for and a wrap to the lower
; limit is made (assuming step = up)
; If the OWB setting is set to 2 (bounce) then once
; a limit is reach the step is inverted
; Inputs:
;     iy = channel buffer
; ** all other registers are preserved, except 'a'
; -------------------------------------------------
ProcessToneOWB:
   push ix
   push hl
   push de
   push bc

   ; find direction
   ld a, (iy+CH_TONE_STEP+1)
   and a, 2
   cp a, 0 ; stepping up
   jp nz, PTOWB_DOWN
PTOWB_UP:

   ; how far are we from the upper limit?
   ld l, (iy+CH_TONE_ULIMIT)
   ld h, (iy+CH_TONE_ULIMIT+1)
   ld c, (iy+CH_INIT_TONE)
   ld b, (iy+CH_INIT_TONE+1)
   scf
   ccf
   sbc hl,bc
   push hl ; store result in ix for later
   pop ix

   ; does next step take us over the upper limit
   ld c, (iy+CH_TONE_STEP)
   ld b, (iy+CH_TONE_STEP+1)
   scf
   ccf
   sbc hl,bc
   jr nc, PTOWB_UP_OK
   
   ; at this point the next step increment will take us over the ulimit
   ld l, (iy+CH_TONE_STEP)
   ld h, (iy+CH_TONE_STEP+1)
   push ix
   pop bc
   scf
   ccf
   sbc hl,bc ; hl now=remainder
   
   ; do we wrap or bounce
   ld a, (iy+CH_TONE_OWB)
   cp a, 1
   jr nz, PTOWB_UP_BOUNCE
PTOWB_UP_WRAP:
   push hl
   pop bc
   ld l, (iy+CH_TONE_LLIMIT)
   ld h, (iy+CH_TONE_LLIMIT+1)
   add hl,bc
   ld (iy+CH_INIT_TONE), l
   ld (iy+CH_INIT_TONE+1), h
   jp PTOWB_END
PTOWB_UP_BOUNCE:
   push hl
   pop bc
   ld l, (iy+CH_TONE_ULIMIT)
   ld h, (iy+CH_TONE_ULIMIT+1)
   scf
   ccf
   sbc hl,bc
   ld (iy+CH_INIT_TONE), l
   ld (iy+CH_INIT_TONE+1), h
   ; invert step
   ld hl, 1024
   ld c, (iy+CH_TONE_STEP)
   ld b, (iy+CH_TONE_STEP+1)
   scf
   ccf
   sbc hl,bc
   ld (iy+CH_TONE_STEP),l
   ld (iy+CH_TONE_STEP+1),h
   jp PTOWB_END

PTOWB_UP_OK
   ; we haven't reached the upper limit so just add tone+step
   ld l, (iy+CH_INIT_TONE)
   ld h, (iy+CH_INIT_TONE+1)
   ld c, (iy+CH_TONE_STEP)
   ld b, (iy+CH_TONE_STEP+1)
   add hl,bc
   ld (iy+CH_INIT_TONE), l
   ld (iy+CH_INIT_TONE+1), h
   jp PTOWB_END
   
PTOWB_DOWN:
   ; how far are we from the lower limit?
   ld l, (iy+CH_INIT_TONE)
   ld h, (iy+CH_INIT_TONE+1)
   ld c, (iy+CH_TONE_LLIMIT)
   ld b, (iy+CH_TONE_LLIMIT+1)
   scf
   ccf
   sbc hl,bc
   push hl ; store result in ix for later
   pop ix
   
   ; invert the step to get a real value
   ld hl, 1024
   ld c, (iy+CH_TONE_STEP)
   ld b, (iy+CH_TONE_STEP+1)
   scf
   ccf
   sbc hl,bc
   push hl ; store result in de for later
   pop de

   ; does the step take us over the limit?
   push ix
   pop hl
   push de
   pop bc
   scf
   ccf
   sbc hl,bc
   jr nc, PTOWB_DOWN_OK

   ; calc distance beyond limit
   push de
   pop hl
   push ix
   pop bc
   scf
   ccf
   sbc hl, bc
   
   ; do we wrap or bounce
   ld a, (iy+CH_TONE_OWB)
   cp a, 1
   jr nz, PTOWB_DOWN_BOUNCE
PTOWB_DOWN_WRAP:
   push hl
   pop bc
   ld l, (iy+CH_TONE_ULIMIT)
   ld h, (iy+CH_TONE_ULIMIT+1)
   scf
   ccf
   sbc hl,bc
   ld (iy+CH_INIT_TONE), l
   ld (iy+CH_INIT_TONE+1), h
   jp PTOWB_END
PTOWB_DOWN_BOUNCE:
   ld c, (iy+CH_TONE_LLIMIT)
   ld b, (iy+CH_TONE_LLIMIT+1)
   add hl,bc
   ld (iy+CH_INIT_TONE), l
   ld (iy+CH_INIT_TONE+1), h
   ; invert step
   ld hl, 1024
   ld c, (iy+CH_TONE_STEP)
   ld b, (iy+CH_TONE_STEP+1)
   scf
   ccf
   sbc hl,bc
   ld (iy+CH_TONE_STEP),l
   ld (iy+CH_TONE_STEP+1),h
   jp PTOWB_END
PTOWB_DOWN_OK:
   ; we haven't reached the upper limit so just add tone+step
   ld l, (iy+CH_INIT_TONE)
   ld h, (iy+CH_INIT_TONE+1)
   ld c, (iy+CH_TONE_STEP)
   ld b, (iy+CH_TONE_STEP+1)
   add hl,bc
   ld (iy+CH_INIT_TONE), l
   ld (iy+CH_INIT_TONE+1), h
   jp PTOWB_END

PTOWB_END:
   ; AND tone with 03ffh to keep it in bounds
   ld a, (iy+CH_INIT_TONE+1)
   and a, 3
   ld (iy+CH_INIT_TONE+1), a

   pop bc
   pop de
   pop hl
   pop ix
   ret



; -------------------------------------------------
; ProcessVolOWB
; If the OWB setting is set to 1 (wrap) then the
; upper limit is checked for and a wrap to the lower
; limit is made (assuming step = up)
; If the OWB setting is set to 2 (bounce) then once
; a limit is reach the step is inverted
; Inputs:
;     iy = channel buffer
; ** all other registers are preserved, except 'a'
; -------------------------------------------------
ProcessVolOWB:
   push de
   push bc

   ; find direction
   ld a, (iy+CH_VOL_STEP)
   and a, 8
   cp a, 0 ; stepping up
   jp nz, PVOWB_DOWN
PVOWB_UP:

   ; how far are we from the upper limit?
   ld a, (iy+CH_VOL_ULIMIT)
   ld c, (iy+CH_INIT_VOL)
   sub a,c
   ld b,a ; store result in b for later

   ; does next step take us over the upper limit
   ld c, (iy+CH_VOL_STEP)
   scf
   ccf
   sbc a,c
   jr nc, PVOWB_UP_OK
   
   ; at this point the next step increment will take us over the ulimit
   ld a, (iy+CH_VOL_STEP)
   ld c,b
   sub a,c ; a now=remainder
   ld d,a
   
   ; do we wrap or bounce
   ld a, (iy+CH_VOL_OWB)
   cp a, 1
   jr nz, PVOWB_UP_BOUNCE
PVOWB_UP_WRAP:
   ld a, (iy+CH_VOL_LLIMIT)
   add a, d
   ld (iy+CH_INIT_VOL), a
   jp PVOWB_END
PVOWB_UP_BOUNCE:
   ld a, (iy+CH_VOL_ULIMIT)
   sub a, d
   ld (iy+CH_INIT_VOL), a
   ; invert step
   ld a, 16
   ld c, (iy+CH_VOL_STEP)
   sub a, c
   ld (iy+CH_VOL_STEP),a
   jp PVOWB_END

PVOWB_UP_OK
   ; we haven't reached the upper limit so just add vol+step
   ld a, (iy+CH_INIT_VOL)
   ld c, (iy+CH_VOL_STEP)
   add a,c
   ld (iy+CH_INIT_VOL), a
   jp PVOWB_END
   
PVOWB_DOWN:
   ; how far are we from the lower limit?
   ld a, (iy+CH_INIT_VOL)
   ld c, (iy+CH_VOL_LLIMIT)
   sub a,c
   ld b,a ; store result in b for later
   
   ; invert the step to get a real value
   ld a, 16
   ld c, (iy+CH_VOL_STEP)
   sub a,c
   ld d,a ; store result in de for later

   ; does the step take us over the limit?
   ld c, d
   ld a, b
   scf
   ccf
   sbc a,c 
   jr nc, PVOWB_DOWN_OK

   ; calc distance beyond limit
   ld a, d
   ld c, b
   sub a, c
   ld e, a

   ; do we wrap or bounce
   ld a, (iy+CH_VOL_OWB)
   cp a, 1
   jr nz, PVOWB_DOWN_BOUNCE
PVOWB_DOWN_WRAP:
   ld a, (iy+CH_VOL_ULIMIT)
   ld c, e
   sub a,c
   ld (iy+CH_INIT_VOL), a
   jp PVOWB_END
PVOWB_DOWN_BOUNCE:
   ld a, (iy+CH_VOL_LLIMIT)
   ld c, e
   add a,c
   ld (iy+CH_INIT_VOL), a
   ; invert step
   ld a, 16
   ld c, (iy+CH_VOL_STEP)
   sub a,c
   ld (iy+CH_VOL_STEP),a
   jp PVOWB_END
PVOWB_DOWN_OK:
   ; we haven't reached the upper limit so just add vol+step
   ld a, (iy+CH_INIT_VOL)
   ld c, (iy+CH_VOL_STEP)
   add a,c
   ld (iy+CH_INIT_VOL), a
   jp PVOWB_END

PVOWB_END:
   ; AND vol with 015h to keep it in bounds
   ld a, (iy+CH_INIT_VOL)
   and a, 15
   ld (iy+CH_INIT_VOL), a

   pop bc
   pop de
   ret


   


; -------------------------------------------------
; DoStopChannel
; Stops the channel playing by cutting the volume and
; resetting the running flag of the specified channel 
; Inputs a = channel to stop
; -------------------------------------------------
DoStopChannel:
   push ix
   push iy
   push de
   push bc

   and a, 3    ; ensure in range also noise4 = 0

DSC_SWITCH_A
   cp a, 0
   jr nz, DSC_CASE_1
   ld ix, RIGHT_CH_CTRL
   ld b, NOISE_VOL
   ld iy, NoiseBuf
   jr DSC_END_SWITCH
DSC_CASE_1
   cp a, 1
   jr nz, DSC_CASE_2
   ld ix, LEFT_CH_CTRL
   ld b, TONE_VOL1
   ld iy, Ch1Buf
   jr DSC_END_SWITCH
DSC_CASE_2
   cp a, 2
   jr nz, DSC_CASE_3
   ld ix, LEFT_CH_CTRL
   ld b, TONE_VOL2
   ld iy, Ch2Buf
   jr DSC_END_SWITCH
DSC_CASE_3
   ld ix, LEFT_CH_CTRL
   ld b, TONE_VOL3
   ld iy, Ch3Buf
DSC_END_SWITCH
	
   ; shut off volume
   ld a, b
   or a, FIRST_TXFER
   or a, 15
   ld (ix), a

   ; stop playing
   ld a, 0
   ld (iy+CH_RUNNING), a

   pop bc
   pop de
   pop iy
   pop ix
   ret

; the internal buffers for each channel
BUFFERS:
NoiseBuf	equ BUFFERS
Ch1Buf		equ BUFFERS+40h
Ch2Buf		equ BUFFERS+80h
Ch3Buf		equ BUFFERS+0C0h
Presets		equ BUFFERS+100h
