REG_Z80SND      equ     0b8h    ; 16bit w - switch the z80 and/or snd chip on or off
ZS_Z0S0         equ     0aaaah  ; Z80 off Snd off
ZS_Z0S1         equ     0aa55h  ; Z80 off Snd on - Snd with direct access from TLCS
ZS_Z1S0         equ     055aah  ; Z80 on Snd off
ZS_Z1S1         equ     05555h  ; Z80 on Snd on - Snd only available via Z80

REG_Z80_NMI     equ     0bah    ; 8bit w - triggers Z80 NMI
REG_Z80_COMM    equ     0bch    ; 8bit r/w - comms with z80 address 0x8000

_Z80RAM         equ     7000h   ; start of Z80 RAM


        RAMDD   Z80_GROUP_INFO
        RAMDD   Z80_HOST_TABLE_START

; --------------------------------------------------------------------
; NeoTracker_SoundInit - PUBLIC
; Comment:      Installs sound driver and sets timer to fire z80 ints
; Parameters:   None
; --------------------------------------------------------------------
NeoTracker_SoundInit:
        push bc
        push xde
        push xhl

        ; copy sound driver to z80 ram
        ldw (REG_Z80SND), ZS_Z0S0  ; turn off z80 and snd chip
        ld xde, _Z80RAM
        ld xhl, NeoTracker_SoundDriver
        ld bc, (xhl+08fh)   ; get sound driver size
        ldir (xde+), (xhl+) ; copy sound driver
        ldw (REG_Z80SND), ZS_Z1S1   ; restart z80
        ld (Z80_GROUP_INFO), xde
        add xde, 6
        ld (Z80_HOST_TABLE_START), xde

        ; set up timer for im1 z80 interrupt
        and (20h), 0f7h     ; turn off timer 3
        and (28h), 033h     ; set timer 2,3 to 8 bit timers
        or (28h), 04h       ; timer 3 internal clock = t1 (1.302uS)
        and (25h), 0fh      ; clear timer 3 settings
        or (25h), 0b0h      ; clear flipflip and flip with timer 3
        ld (27h), 62h       ; set comparitor to interrupt z80 about 7800/sec
        ld (20h), 88h       ; turn timer 3 back on

        pop xhl
        pop xde
        pop bc
        ret


; --------------------------------------------------------------------
; NeoTracker_PlayMusic - PUBLIC
; Comments:     Sends tune number to the driver
; Parameters:   a = tune number 
; *NOTE*        No error checking is done to ensure group is valid
;               be careful this could cause it to crash
; --------------------------------------------------------------------
NeoTracker_PlayMusic:
        push xwa
        
        sll 2, a
        or a, 3

        ; stop any sound playing
        ld (REG_Z80_COMM), a
        ld (REG_Z80_NMI), 1
                
        pop xwa

        ret


; --------------------------------------------------------------------
; NeoTracker_PlaySFX - PUBLIC
; Comments:     Sends sound effect number to the driver - note that this
;               will temporarily stop any channels currently playing 
;               music until the sfx has completed.
; Parameters:   a = tune number 
; *NOTE*        No error checking is done to ensure group is valid
;               be careful this could cause it to crash
; --------------------------------------------------------------------
NeoTracker_PlaySFX:
        push xwa
        
        sll 2, a
        or a, 2

        ; stop any sound playing
        ld (REG_Z80_COMM), a
        ld (REG_Z80_NMI), 1
                
        pop xwa

        ret



; --------------------------------------------------------------------
; NeoTracker_SendGroup - PUBLIC
; Comments:     Sends specified group to sound driver in Raw format
; Parameters:   bc - size of group
;               xhl - pointer to group
; *NOTE*        No error checking is done to ensure group is valid
;               be careful this could cause it to crash
; --------------------------------------------------------------------
NeoTracker_SendGroup:
	push xwa
	push xbc
	push xde
	push xhl

	; stop any sound playing
	ld (REG_Z80_COMM), 0ffh
	ld (REG_Z80_NMI), 1

	; wait a while for the driver to ack the stop command
	push c
	ld c, 100
SGR_Stop_Wait:
	ld a, (REG_Z80_COMM)
	cp a, 0
	jr z, SGR_Stop_Complete
	djnz c, SGR_Stop_Wait

	calr NeoTracker_SoundInit       ; it didn't acknowledge so reinstall it
SGR_Stop_Complete:
	pop c

	; copy raw group information to z80 ram
	ld xde, (Z80_GROUP_INFO)	; set destination normally 7ab5h
	ldir (xde+), (xhl+)

	; wait for ack
	ld a, (70deh)
SGR_Wait1:
	cp a, (70deh)
	jr z, SGR_Wait1

	pop xhl
	pop xde
	pop xbc
	pop xwa
	ret


; --------------------------------------------------------------------
;
; Z80 Sound Driver - PRIVATE
;
; --------------------------------------------------------------------
NeoTracker_SoundDriver:
	DB	0f3h,031h,0c0h,000h,0c3h,0e6h,001h,000h
	DB	0e5h,0cdh,0c2h,008h,0e1h,0c9h,000h,000h
	DB	07dh,02fh,06fh,07ch,02fh,067h,023h,0c9h
	DB	05dh,054h,013h,0afh,077h,0edh,0b0h,0c9h
	DB	021h,0dch,000h,07eh,0d6h,004h,0c9h,000h
	DB	085h,06fh,0d0h,024h,0c9h,000h,000h,000h
	DB	0c3h,08fh,004h,0cah,031h,00eh,000h,000h
	DB	0f3h,008h,03ch,0d3h,0ffh,008h,0fbh,0edh
	DB	04dh,011h,000h,000h,048h,042h,0b7h,0c8h
	DB	01fh,030h,003h,0ebh,009h,0ebh,0cbh,021h
	DB	0cbh,010h,018h,0f2h,047h,03ah,0e5h,001h
	DB	0cdh,041h,000h,0ebh,029h,029h,07ch,0c3h
	DB	075h,004h,000h,000h,000h,000h,0f5h,0e5h
	DB	0d5h,021h,0d1h,000h,0cdh,0c7h,004h,011h
	DB	000h,080h,01ah,077h,0eeh,0ffh,012h,0b7h
	DB	0cch,02ch,006h,0d1h,0e1h,0f1h,0edh,045h
	DB	04bh,031h,00bh,030h,028h,078h,00ah,0ffh
	DB	020h,020h,020h,042h,0ffh,0ffh,000h,0b5h
	DB	00ah,000h,000h,000h,000h,000h,000h,000h
	DB	000h,000h,000h,000h,000h,000h,000h,000h
	DB	000h,000h,000h,000h,000h,000h,000h,000h
	DB	000h,000h,000h,000h,000h,000h,000h,000h
	DB	000h,000h,000h,000h,000h,000h,000h,000h
	DB	000h,000h,000h,000h,000h,000h,000h,000h
	DB	000h,000h,000h,000h,000h,000h,000h,000h
	DB	000h,000h,000h,000h,000h,000h,000h,000h
	DB	000h,000h,000h,000h,000h,000h,000h,000h
	DB	000h,000h,000h,000h,000h,000h,000h,000h
	DB	000h,000h,000h,000h,000h,000h,000h,000h
	DB	000h,000h,000h,000h,000h,000h,000h,000h
	DB	000h,000h,000h,000h,000h,000h,000h,000h
	DB	000h,000h,000h,000h,000h,000h,000h,000h
	DB	000h,000h,000h,000h,000h,000h,000h,000h
	DB	000h,000h,000h,000h,000h,000h,000h,000h
	DB	000h,000h,000h,000h,000h,000h,000h,000h
	DB	000h,000h,000h,000h,000h,000h,000h,000h
	DB	000h,000h,000h,000h,000h,000h,000h,000h
	DB	000h,000h,000h,000h,000h,000h,000h,000h
	DB	000h,000h,000h,000h,000h,000h,000h,000h
	DB	000h,000h,000h,000h,000h,000h,000h,000h
	DB	000h,000h,000h,000h,000h,000h,000h,000h
	DB	000h,000h,000h,000h,000h,000h,000h,000h
	DB	000h,000h,000h,000h,000h,000h,000h,000h
	DB	000h,000h,000h,000h,000h,000h,000h,000h
	DB	000h,000h,000h,000h,000h,000h,000h,000h
	DB	000h,000h,000h,000h,000h,000h,000h,000h
	DB	000h,000h,000h,000h,000h,000h,000h,000h
	DB	000h,000h,000h,000h,000h,000h,000h,000h
	DB	000h,000h,000h,000h,000h,000h,000h,000h
	DB	000h,000h,000h,000h,000h,000h,000h,000h
	DB	000h,000h,000h,000h,000h,000h,000h,000h
	DB	000h,000h,000h,000h,000h,000h,000h,000h
	DB	000h,000h,000h,000h,000h,000h,000h,000h
	DB	000h,000h,000h,000h,000h,000h,000h,000h
	DB	000h,000h,000h,000h,000h,000h,000h,000h
	DB	000h,000h,000h,000h,000h,000h,000h,000h
	DB	000h,000h,000h,000h,000h,000h,000h,000h
	DB	000h,000h,000h,000h,000h,000h,000h,000h
	DB	000h,000h,000h,000h,000h,000h,000h,000h
	DB	000h,000h,000h,000h,000h,000h,000h,000h
	DB	000h,000h,000h,000h,000h,000h,0edh,056h
	DB	02eh,001h,07dh,0cfh,07dh,0c6h,020h,06fh
	DB	007h,030h,0f7h,021h,0c0h,000h,001h,025h
	DB	001h,0dfh,021h,0d7h,000h,036h,078h,008h
	DB	0afh,008h,0d3h,0ffh,0fbh,032h,000h,0c0h
	DB	021h,0d0h,000h,03ah,0d1h,000h,0beh,028h
	DB	007h,0cdh,0c7h,004h,07eh,0cdh,0fah,004h
	DB	0f3h,008h,04fh,008h,0fbh,021h,0deh,000h
	DB	07eh,091h,028h,0e4h,0edh,044h,071h,023h
	DB	077h,021h,0d4h,000h,01eh,000h,001h,0d2h
	DB	000h,0cdh,04ah,002h,021h,0d7h,000h,01eh
	DB	004h,001h,0d3h,000h,0cdh,04ah,002h,03ah
	DB	0dfh,000h,047h,0afh,0cfh,0cdh,0e3h,005h
	DB	018h,0beh,00ah,0b7h,0c8h,04fh,0e5h,0d9h
	DB	0e1h,03ah,0dfh,000h,087h,087h,046h,0cdh
	DB	041h,000h,023h,04eh,023h,046h,0ebh,009h
	DB	001h,0e2h,004h,0b7h,0edh,042h,030h,001h
	DB	009h,0ebh,072h,02bh,073h,0d9h,0d8h,021h
	DB	0a5h,002h,022h,085h,002h,069h,060h,006h
	DB	004h,016h,001h,0edh,053h,0dch,000h,0d5h
	DB	0c5h,0cdh,096h,002h,0cdh,0a5h,002h,0c1h
	DB	0d1h,01ch,0cbh,002h,010h,0edh,0c9h,021h
	DB	016h,006h,01eh,000h,018h,0dch,03ah,0dch
	DB	000h,006h,020h,0cdh,041h,000h,0fdh,021h
	DB	0e5h,000h,0fdh,019h,0c9h,0cbh,01dh,0d0h
	DB	0cdh,0b3h,002h,0e5h,0cdh,0f9h,002h,0e1h
	DB	0afh,018h,002h,03eh,001h,0d9h,0fdh,0cbh
	DB	008h,046h,021h,003h,000h,020h,003h,021h
	DB	00ch,000h,0fdh,0e5h,0d1h,019h,011h,0e0h
	DB	000h,0b7h,020h,001h,0ebh,001h,005h,000h
	DB	0edh,0b0h,0d9h,0c9h,057h,02ah,0e2h,000h
	DB	019h,0ebh,0cdh,0dch,005h,0cdh,0b0h,002h
	DB	0fdh,0cbh,008h,086h,0afh,0fdh,077h,001h
	DB	018h,0c9h,021h,000h,000h,022h,0e0h,000h
	DB	0cdh,0b0h,002h,0fdh,0cbh,008h,0c6h,018h
	DB	0ebh,0fdh,0e5h,0e1h,07eh,0b7h,028h,008h
	DB	035h,020h,005h,00eh,000h,0cdh,07bh,003h
	DB	023h,07eh,0d6h,001h,077h,0d0h,03eh,001h
	DB	0f7h,04fh,0e5h,021h,0e4h,000h,0cdh,079h
	DB	004h,047h,0cbh,019h,030h,007h,0cbh,042h
	DB	028h,003h,006h,000h,070h,0e1h,070h,0e5h
	DB	0cdh,07fh,003h,0e1h,018h,0dbh,03eh,002h
	DB	0f7h,0fdh,046h,018h,0feh,003h,028h,022h
	DB	057h,03eh,00fh,0a0h,04fh,03eh,0f0h,0a0h
	DB	047h,0cbh,01ah,038h,008h,03eh,003h,0f7h
	DB	087h,087h,087h,087h,047h,0cbh,01ah,079h
	DB	038h,003h,03eh,004h,0f7h,0b0h,047h,0fdh
	DB	070h,018h,03eh,001h,0f7h,0fdh,04eh,011h
	DB	020h,007h,03eh,004h,0f7h,0fdh,077h,011h
	DB	04fh,0fdh,0e5h,0e1h,03eh,002h,0efh,0cdh
	DB	079h,004h,0fdh,086h,000h,0fdh,077h,000h
	DB	079h,0b7h,0c8h,016h,003h,018h,051h,030h
	DB	0adh,03eh,004h,0f7h,0feh,00fh,0cah,001h
	DB	004h,00eh,0ffh,057h,087h,082h,021h,0e6h
	DB	003h,0efh,07eh,0b7h,028h,001h,0f7h,047h
	DB	023h,07ah,0feh,004h,038h,006h,07eh,023h
	DB	066h,06fh,078h,0e9h,056h,023h,07eh,0b7h
	DB	028h,026h,0fdh,0e5h,0e1h,0efh,0cbh,079h
	DB	028h,01dh,07ah,0feh,002h,020h,00dh,04ah
	DB	0ebh,0fdh,06eh,00ah,0fdh,066h,00bh,078h
	DB	0efh,046h,0ebh,051h,03ah,0dah,000h,0b7h
	DB	028h,004h,03eh,005h,0bah,0c8h,070h,046h
	DB	0d9h,0e7h,030h,00bh,03ah,0d3h,000h,023h
	DB	0a6h,028h,002h,0d9h,0c9h,02bh,07eh,0d9h
	DB	00fh,00fh,00fh,0b2h,0cfh,0c9h,005h,002h
	DB	012h,004h,005h,015h,004h,006h,016h,007h
	DB	004h,017h,005h,02eh,004h,000h,041h,004h
	DB	008h,072h,004h,00bh,0d4h,002h,000h,0eah
	DB	002h,0fdh,0e5h,0cdh,00bh,004h,0fdh,0e1h
	DB	0f1h,0f1h,0c9h,0cdh,016h,006h,03ah,0d2h
	DB	000h,021h,0ddh,000h,0a6h,0c8h,0e7h,0cdh
	DB	099h,002h,021h,0e7h,003h,006h,004h,00eh
	DB	000h,0e5h,0c5h,0cdh,0a4h,003h,0c1h,0e1h
	DB	03eh,003h,0efh,010h,0f4h,0c9h,04fh,0b7h
	DB	020h,002h,00eh,0ffh,0cdh,063h,004h,034h
	DB	0ebh,071h,023h,011h,0e2h,000h,0ebh,018h
	DB	018h,0cdh,063h,004h,0ebh,001h,0fdh,0ffh
	DB	009h,07eh,0feh,0ffh,028h,006h,03dh,020h
	DB	003h,0ebh,035h,0c9h,077h,023h,011h,0e2h
	DB	000h,001h,002h,000h,0edh,0b0h,0afh,032h
	DB	0e1h,000h,0c9h,0fdh,0e5h,0e1h,03eh,019h
	DB	0efh,05dh,054h,07eh,087h,086h,023h,0efh
	DB	0ebh,0c9h,032h,0e5h,001h,032h,0d4h,000h
	DB	0c9h,03eh,001h,0f7h,028h,004h,07eh,016h
	DB	001h,0c9h,03ch,0f7h,028h,002h,03eh,004h
	DB	0c6h,004h,0f7h,077h,016h,000h,0c9h,0feh
	DB	009h,038h,00ah,0f5h,03eh,008h,0cdh,09dh
	DB	004h,05fh,0f1h,0d6h,008h,0d9h,0e6h,00fh
	DB	047h,02ah,0e0h,000h,07ch,026h,000h,0b7h
	DB	020h,00dh,0edh,05bh,0e2h,000h,01ah,013h
	DB	0edh,053h,0e2h,000h,06fh,03eh,008h,0cbh
	DB	025h,0cbh,014h,03dh,010h,0e9h,054h,067h
	DB	022h,0e0h,000h,07ah,0d9h,0b7h,0c9h,07eh
	DB	034h,0e6h,00fh,021h,0c0h,000h,0efh,0c9h
	DB	03eh,0ffh,01eh,004h,021h,084h,005h,04fh
	DB	03ah,0d3h,000h,047h,0c3h,072h,002h,0feh
	DB	086h,028h,0edh,00fh,0e6h,01fh,021h,0bbh
	DB	00ah,087h,0efh,07eh,023h,046h,021h,0f6h
	DB	004h,087h,0efh,0c3h,09eh,003h,0fah,005h
	DB	054h,000h,0b7h,0fah,0dfh,004h,00fh,038h
	DB	010h,00fh,030h,0ceh,02ah,0b9h,00ah,011h
	DB	0d3h,000h,0e6h,01fh,047h,00eh,004h,018h
	DB	011h,0f5h,0cdh,08fh,002h,0f1h,00fh,0d0h
	DB	047h,09fh,02ah,0b7h,00ah,011h,0d2h,000h
	DB	00eh,000h,032h,0c3h,005h,0edh,053h,099h
	DB	005h,078h,0e6h,01fh,028h,00bh,047h,05eh
	DB	023h,07eh,0e6h,03fh,057h,02bh,019h,010h
	DB	0f6h,023h,07eh,0e6h,0c0h,007h,007h,023h
	DB	03ch,047h,0c5h,0e5h,0ddh,0e1h,03eh,002h
	DB	0cdh,041h,000h,0ddh,019h,0c1h,05eh,023h
	DB	056h,03eh,03fh,0a2h,057h,0c5h,0d5h,07eh
	DB	023h,0e6h,0c0h,007h,007h,0f5h,081h,0cdh
	DB	099h,002h,0f1h,00eh,001h,047h,0b7h,028h
	DB	004h,0cbh,001h,010h,0fch,0d1h,0cdh,096h
	DB	005h,0c1h,010h,0dah,03eh,004h,0b9h,0c8h
	DB	021h,0d4h,000h,036h,078h,023h,0afh,077h
	DB	023h,077h,018h,076h,0cbh,01ch,0d0h,07dh
	DB	007h,038h,005h,0fdh,07eh,009h,0bdh,0c0h
	DB	0e5h,0cdh,00bh,004h,0e1h,0c9h,079h,0d9h
	DB	021h,0d2h,000h,0a6h,0d9h,028h,00ah,03ah
	DB	0c3h,005h,0fdh,0beh,009h,028h,002h,030h
	DB	02bh,079h,0d9h,0b6h,077h,0fdh,0e5h,0e1h
	DB	001h,01fh,000h,0dfh,0d9h,0ebh,019h,0ebh
	DB	0fdh,073h,005h,0fdh,072h,006h,0fdh,0cbh
	DB	008h,0c6h,03eh,000h,0fdh,077h,009h,007h
	DB	0d4h,0d5h,005h,0ddh,0e5h,0d1h,0fdh,073h
	DB	00ah,0fdh,072h,00bh,0c9h,011h,033h,000h
	DB	0fdh,0cbh,008h,086h,0fdh,073h,00eh,0fdh
	DB	072h,00fh,0c9h,021h,0dbh,000h,07eh,0b7h
	DB	0c8h,035h,0c0h,02bh,07eh,023h,077h,021h
	DB	005h,006h,0cdh,092h,002h,03ah,0d2h,000h
	DB	0b7h,0c0h,006h,011h,0cdh,041h,000h,053h
	DB	0edh,053h,0dah,000h,0c9h,0fdh,07eh,015h
	DB	03ch,0feh,00fh,028h,009h,0fdh,077h,015h
	DB	047h,016h,005h,0c3h,0d0h,003h,016h,001h
	DB	0cdh,0d0h,003h,0d9h,0e7h,011h,0d2h,000h
	DB	038h,001h,013h,023h,07eh,0ebh,0eeh,0ffh
	DB	0a6h,077h,0d9h,0c9h,021h,000h,000h,0e5h
	DB	0edh,045h,000h,000h,000h,000h,000h,000h
	DB	000h,000h,000h,000h,000h,000h,000h,000h
	DB	000h,000h,000h,000h,000h,000h,000h,000h
	DB	000h,000h,000h,000h,000h,000h,000h,000h
	DB	000h,000h,000h,000h,000h,000h,000h,000h
	DB	000h,000h,000h,000h,000h,000h,000h,000h
	DB	000h,000h,000h,000h,000h,000h,000h,000h
	DB	000h,000h,000h,000h,000h,000h,000h,000h
	DB	000h,000h,000h,000h,000h,000h,000h,000h
	DB	000h,000h,000h,000h,000h,000h,000h,000h
	DB	000h,000h,000h,000h,000h,000h,000h,000h
	DB	000h,000h,000h,000h,000h,000h,000h,000h
	DB	000h,000h,000h,000h,000h,000h,000h,000h
	DB	000h,000h,000h,000h,000h,000h,000h,000h
	DB	000h,000h,000h,000h,000h,000h,000h,000h
	DB	000h,000h,000h,000h,000h,000h,000h,000h
	DB	000h,000h,000h,000h,000h,000h,000h,000h
	DB	000h,000h,000h,000h,000h,0bch,005h,06ah
	DB	005h,01ch,005h,0d2h,004h,08ch,004h,04ch
	DB	004h,00eh,004h,0d4h,003h,09ch,003h,069h
	DB	003h,038h,003h,009h,003h,00fh,006h,005h
	DB	004h,003h,002h,001h,000h,000h,010h,020h
	DB	030h,040h,050h,060h,0f0h,000h,001h,002h
	DB	004h,00ch,000h,00ch,00fh,00dh,00bh,009h
	DB	007h,005h,0ffh,0ffh,043h,0d7h,003h,0b5h
	DB	036h,098h,0ddh,021h,034h,006h,0e6h,060h
	DB	032h,032h,006h,0c8h,011h,022h,000h,0ddh
	DB	019h,0d6h,020h,020h,0fah,0c9h,0cdh,05eh
	DB	007h,05ch,087h,021h,0bdh,006h,0efh,07bh
	DB	05eh,023h,056h,0b7h,0c8h,0cbh,03ah,0cbh
	DB	01bh,03dh,020h,0f9h,0c9h,0edh,044h,06fh
	DB	087h,09fh,067h,0d8h,029h,0c9h,03ah,032h
	DB	006h,0feh,060h,0c0h,0f6h,083h,0cbh,061h
	DB	028h,002h,0f6h,004h,032h,000h,040h,0c9h
	DB	0feh,00fh,0d8h,03eh,00fh,0c9h,021h,000h
	DB	000h,03eh,010h,029h,0ebh,0edh,06ah,0ebh
	DB	030h,004h,009h,030h,001h,01ch,03dh,020h
	DB	0f2h,053h,05ch,0c9h,01ah,013h,06fh,026h
	DB	000h,029h,029h,029h,029h,0e6h,00fh,06fh
	DB	0c9h,01ah,013h,04fh,007h,007h,007h,0e6h
	DB	007h,0efh,07eh,0c9h,0d5h,0ddh,05eh,000h
	DB	0ddh,056h,001h,0ddh,06eh,002h,0ddh,066h
	DB	003h,03ah,0bch,006h,019h,03dh,020h,0fch
	DB	07ch,0aah,0fah,0b2h,007h,07ch,0b7h,0fah
	DB	098h,007h,0b9h,038h,01dh,079h,018h,006h
	DB	079h,0edh,044h,0bch,038h,014h,067h,02eh
	DB	000h,0cbh,048h,020h,00ch,0cbh,050h,020h
	DB	009h,0ebh,0d7h,0ebh,0cdh,00eh,008h,018h
	DB	001h,0d7h,0d1h,0ddh,075h,002h,0ddh,074h
	DB	003h,07ch,0c9h,021h,000h,000h,087h,0c8h
	DB	0f5h,0c5h,0d5h,0ebh,00fh,02fh,03ch,06fh
	DB	0e6h,003h,05fh,07dh,00fh,00fh,0e6h,007h
	DB	028h,004h,0c6h,002h,018h,003h,07bh,03dh
	DB	05ah,021h,0f2h,006h,019h,019h,05eh,023h
	DB	056h,0b7h,028h,007h,0cbh,03ah,0cbh,01bh
	DB	03dh,020h,0f9h,006h,000h,0cdh,046h,007h
	DB	0ebh,0d1h,0c1h,0f1h,0d0h,0d7h,0c9h,0afh
	DB	0ddh,077h,003h,0ddh,077h,002h,0ddh,07eh
	DB	007h,0ddh,077h,008h,0ddh,07eh,006h,0ddh
	DB	04eh,004h,0cdh,0bbh,007h,0ebh,0ddh,073h
	DB	000h,0ddh,072h,001h,0c9h,0cbh,048h,020h
	DB	001h,04fh,057h,0ddh,07eh,001h,0e6h,080h
	DB	0ddh,0b6h,006h,0cdh,00ah,008h,07ah,0cbh
	DB	048h,0c8h,04fh,0c9h,0ddh,0e5h,0ddh,019h
	DB	05fh,0ddh,07eh,006h,0b7h,028h,00ah,0ddh
	DB	07eh,008h,0b7h,028h,008h,03dh,0ddh,077h
	DB	008h,0afh,04fh,018h,020h,07bh,0cbh,040h
	DB	028h,005h,0cdh,015h,008h,018h,003h,0ddh
	DB	04eh,004h,0c5h,0ddh,046h,005h,0cdh,074h
	DB	007h,0cbh,040h,028h,007h,0b7h,079h,0f2h
	DB	064h,008h,0edh,044h,0c1h,0ddh,056h,004h
	DB	0ddh,06eh,005h,0ddh,0e1h,0cbh,05dh,0c8h
	DB	067h,07ah,0cbh,07dh,020h,002h,0edh,044h
	DB	084h,0c9h,011h,006h,000h,0ddh,0e5h,0ddh
	DB	019h,001h,03fh,000h,0cdh,074h,007h,0ddh
	DB	0e1h,0a7h,0f2h,08eh,008h,0afh,047h,0ddh
	DB	07eh,00ah,03dh,028h,009h,03dh,028h,00ch
	DB	078h,0b7h,028h,01bh,018h,01ch,078h,0b9h
	DB	038h,018h,018h,007h,0ddh,04eh,00fh,079h
	DB	0b8h,038h,00fh,041h,0ddh,036h,008h,000h
	DB	0ddh,070h,009h,0ddh,07eh,00ah,03ch,0cdh
	DB	090h,009h,03eh,03fh,090h,00fh,00fh,0e6h
	DB	00fh,0c9h,06fh,0cdh,0fah,006h,07dh,0e6h
	DB	00fh,087h,021h,0d3h,008h,0efh,07eh,023h
	DB	066h,06fh,0e9h,024h,00ah,014h,00ah,0e1h
	DB	008h,03eh,009h,0c1h,009h,010h,00ah,006h
	DB	00ah,02ah,0b5h,00ah,078h,087h,087h,087h
	DB	080h,0efh,0ebh,021h,0eah,006h,0cdh,069h
	DB	007h,0ddh,077h,005h,079h,0e6h,00fh,0ddh
	DB	077h,00bh,0cdh,05ch,007h,0ddh,074h,00ch
	DB	087h,087h,0ddh,077h,00fh,0cdh,05ch,007h
	DB	0ddh,074h,00dh,0ddh,075h,00eh,0cdh,02eh
	DB	007h,001h,010h,000h,0cdh,01ah,009h,001h
	DB	009h,000h,0ddh,009h,01ah,0e6h,080h,047h
	DB	01ah,013h,0e6h,07fh,0ddh,077h,004h,021h
	DB	0e5h,006h,0cdh,069h,007h,0b0h,0ddh,077h
	DB	005h,079h,0e6h,01fh,0b0h,0ddh,077h,006h
	DB	01ah,013h,0ddh,077h,007h,0c9h,03eh,00fh
	DB	091h,04fh,0feh,00fh,020h,009h,0ddh,07eh
	DB	00ah,0b7h,0c8h,03eh,004h,018h,041h,0ddh
	DB	071h,002h,03ah,032h,006h,0feh,060h,078h
	DB	020h,002h,0c6h,030h,0ddh,077h,000h,0ddh
	DB	036h,001h,000h,0cdh,00eh,007h,0cdh,0e3h
	DB	009h,0ddh,0e5h,011h,010h,000h,0ddh,019h
	DB	0cdh,0f7h,007h,011h,009h,000h,0ddh,019h
	DB	0cdh,0f7h,007h,0ddh,0cbh,005h,056h,0ddh
	DB	0e1h,028h,004h,0ddh,0cbh,005h,0f6h,0afh
	DB	0ddh,077h,008h,0ddh,077h,009h,03eh,001h
	DB	0ddh,077h,00ah,0b7h,0c8h,05fh,0ddh,0e5h
	DB	016h,000h,0ddh,019h,0feh,002h,038h,002h
	DB	016h,080h,0ddh,07eh,00ah,0ddh,0e1h,087h
	DB	020h,009h,07bh,01ch,0feh,002h,028h,0e6h
	DB	0afh,018h,002h,03ch,0b2h,00eh,03fh,0cdh
	DB	0bbh,007h,0ddh,075h,006h,0ddh,074h,007h
	DB	0c9h,078h,0d6h,040h,087h,0ddh,077h,001h
	DB	021h,000h,000h,0b7h,028h,005h,0ebh,0cdh
	DB	025h,007h,019h,04dh,044h,0ddh,07eh,000h
	DB	0cdh,00eh,007h,078h,0b1h,028h,004h,004h
	DB	0cdh,046h,007h,021h,001h,040h,03ah,032h
	DB	006h,0feh,060h,020h,003h,02dh,03eh,040h
	DB	04fh,07bh,0e6h,00fh,0f6h,080h,0b1h,0ebh
	DB	029h,029h,029h,029h,0ebh,05fh,07ah,0e6h
	DB	07fh,0f3h,073h,077h,0fbh,0c9h,021h,0d5h
	DB	006h,078h,0efh,07eh,0ddh,077h,004h,0c9h
	DB	0ddh,070h,003h,0c9h,0ddh,0e5h,0e1h,001h
	DB	021h,000h,0dfh,001h,00fh,00fh,0ddh,071h
	DB	002h,0c3h,0a4h,00ah,078h,032h,0bch,006h
	DB	0afh,0f5h,0cdh,0fah,006h,0cdh,038h,00ah
	DB	0f1h,0c6h,020h,0feh,080h,038h,0f2h,0c9h
	DB	0ddh,07eh,00ah,0b7h,0c8h,0ddh,046h,005h
	DB	004h,005h,028h,036h,0cbh,080h,011h,010h
	DB	000h,0cdh,02ch,008h,0edh,044h,081h,032h
	DB	033h,006h,0ddh,046h,005h,011h,019h,000h
	DB	0cdh,02ch,008h,0cbh,040h,028h,008h,0f5h
	DB	0edh,044h,081h,032h,033h,006h,0f1h,0cbh
	DB	050h,028h,00fh,0cdh,025h,007h,0cbh,070h
	DB	028h,002h,029h,029h,0ddh,07eh,001h,0cdh
	DB	0cbh,009h,0cdh,07ah,008h,0ddh,086h,002h
	DB	0ddh,086h,003h,0ddh,0cbh,005h,05eh,028h
	DB	009h,047h,03ah,033h,006h,0cdh,05eh,007h
	DB	07ch,080h,047h,0ddh,07eh,004h,0cdh,05eh
	DB	007h,080h,0cdh,040h,007h,04fh,07ch,080h
	DB	0cdh,040h,007h,047h,021h,000h,040h,03ah
	DB	032h,006h,0f6h,090h,0b0h,047h,0e6h,0f0h
	DB	0b1h,077h,02ch,070h,0c9h

